==> taboola_sdk-0.2.1.tar.gz: lib/descriptors/device_descriptor.dart <==
import 'dart:io' as dartIo;
import 'dart:ui';
import 'dart:convert';
import 'package:taboola_sdk/utils/logger.dart';
import 'package:sim_info/sim_info.dart' as pluginSimInfo;
import 'package:device_info_plus/device_info_plus.dart';
import 'package:flutter/cupertino.dart';
import "package:system_info/system_info.dart";
import 'package:taboola_sdk/utils/text_utils.dart';
import 'package:flutter/material.dart';
import 'package:taboola_sdk/native_plugin/taboola_sdk_plugin.dart';

class DeviceDescriptor {
  static const _undefinedValue = "undefined";
  static DeviceInfoPlugin _deviceInfoPlugin = DeviceInfoPlugin();

  //These fields will hold references to device data, allowing memory cache of results
  //TODO: If product leaves MVP, caching in app storage will be better for some values.
  static String _model = ""; //the device’s model string
  static var _lng;
  static String _os = ""; //“Android” or “iOS”
  static String _osv = ""; //the OS version
  static String _mem = ""; //the device current available memory
  static List _scr; //screen sized (height and width)

  static Future<String> getDeviceModel() async {
    //Note: Taboola Android SDK runs a process on BuildConfig.Model before returning its value, will be simulated here.
    if (TextUtils.isEmptyOrNull(_model)) {
      if (dartIo.Platform.isAndroid) {
        AndroidDeviceInfo androidDeviceInfo =
            await _deviceInfoPlugin.androidInfo;
        String nativeModel = androidDeviceInfo.model;
        String nativeManufacturer = androidDeviceInfo.manufacturer;

        //Some devices start with
        if (nativeModel.startsWith(nativeManufacturer)) {
          _model = TextUtils.capitalize(nativeModel);
        } else {
          _model = '${TextUtils.capitalize(nativeManufacturer)} $nativeModel';
        }

        //Html escape value
        _model = HtmlEscape().convert(_model);
      }

      //Note: iOS returns model only, as manufacturer is always Epel. .. .
      if (dartIo.Platform.isIOS) {
        IosDeviceInfo iosDeviceInfo = await _deviceInfoPlugin.iosInfo;
        _model = iosDeviceInfo.model;
      }
    }

    return _model;
  }

  static Future<String> getDeviceOS() async {
    if (TextUtils.isEmptyOrNull(_os)) {
      if (dartIo.Platform.isAndroid) {
        _os = "Android";
      }

      if (dartIo.Platform.isIOS) {
        _os = "iOS";
      }
    }
    return _os;
  }

  /// Uses plugin "device_info"
  /// Source: https://stackoverflow.com/questions/45300661/how-to-check-the-device-os-version-from-flutter
  static Future<String> getDeviceSystemVersion() async {
    if (TextUtils.isEmptyOrNull(_osv)) {
      if (dartIo.Platform.isAndroid) {
        AndroidDeviceInfo androidDeviceInfo =
            await _deviceInfoPlugin.androidInfo;
        _osv = androidDeviceInfo.version.release;
      }

      if (dartIo.Platform.isIOS) {
        IosDeviceInfo iosDeviceInfo = await _deviceInfoPlugin.iosInfo;
        _osv = iosDeviceInfo.systemVersion;
      }
    }
    return _osv;
  }

  ///TODO: Restore in the future, requires either ripping just relevant code from Connectivity plugin (preferred solution) or by asking publishers to add two properties, describing usage of location, to their info.plist: 'NSLocationAlwaysAndWhenInUseUsageDescription' & 'NSLocationWhenInUseUsageDescription'.  Source:  https://pub.dev/packages/connectivity
//  /// Uses plugin "connectivity".
//  /// Existing Android/iOS values are: "wifi", "2g", "3g", "4g"
//  ///
//  /// NOTE: Flutter, incl. Connectivity package currently only provide wifi or cell. No '2g/3g/4g/..' resolution.
//  ///       A PR is open for adding the 'networkType' resolution but still wasn't accepted: https://github.com/flutter/plugins/pull/1945
//  ///
//  /// NOTE: Unlike other values here, every query to this method must pull current results as values are dynamic.
//  ///
//  /// Source: https://pub.dev/packages/connectivity
//  /// Source: https://pub.dev/documentation/connectivity/latest/
//  static Future<String> getNetworkConnSpeed() async {
//      var connectivityResult = await (Connectivity().checkConnectivity());
//      if (connectivityResult == ConnectivityResult.mobile) {
//        return "3g"; //Providing Taboola back-end with common value until better resolution is available.
//      } else if (connectivityResult == ConnectivityResult.wifi) {
//        return "wifi";
//      }
//
//      return "offline"; //value taken from iOS, Android SDK does not handle this case.
//  }

  /// Source: https://pub.dev/packages/system_info
  static Future<String> getDeviceFreeMemory() async {
    const int kiloByte = 1024;

    if (TextUtils.isEmptyOrNull(_mem)) {
      int memInteger;

      if (dartIo.Platform.isAndroid) {
        memInteger = SysInfo.getFreeVirtualMemory() ~/ kiloByte;
      } else {
        memInteger =
            0; //FIXME: Hardcoded, current version of library doesn't work on iOS properly.
      }

      _mem = "$memInteger";
    }
    return _mem;
  }

  /// There are two known ways to measure heights, MediaQuery & window.physicalSize.
  /// Both aren't completely accurate as Flutter only draws to most of the screen, unaware of the phones status bar.
  /// Android MediaQuery measurements rely on BuildContext and therefor are less preferable to window.physicalSize.
  ///
  /// UPDATE 29/12/2019: TODO: To reduce complexity, we commented out the MediaQuery method. Post MVP, if we find issues with screen measurements, we can re-introduce this.
  ///
  /// Source - MediaQueryData: https://stackoverflow.com/questions/57052297/flutter-screen-size-calculation
  /// Source - window.physicalSize: https://stackoverflow.com/questions/49553402/flutter-screen-size
  static List getScreenSize() {
    if (_scr == null) {
      double width, height;

      //Direct dart UI measurement
      Size windowSize = window.physicalSize;
      if (windowSize != null) {
        width = windowSize.width;
        height = windowSize.height;
        //Alternative measurements, BuildContext dependant
      }
//      else {
//        MediaQueryData mediaQueryData = MediaQuery.of(context);
//        var pixRatio = mediaQueryData.devicePixelRatio;
//        windowSize = mediaQueryData.size;
//        width = windowSize.width * pixRatio;
//        height = windowSize.height * pixRatio;
//
//        if (dartIo.Platform.isAndroid) {
//          height += AppBar().preferredSize.height;
//        }
//      }

      _scr = [width, height];
    }
    return _scr;
  }

  /// Language data isn't identical between what's currently available on Flutter and native Android/iOS.
  /// Agreed with product and team leader, at the moment, to provide what is possible and try and classify as closely as possible to Android/iOS values.
  ///
  /// NOTE: 'window.locales' returned an informative list ("[en_US, en, he_IL]") when I tried it, but it is officially known to change values for reasons I couldn't determine so not used right now.
  ///
  /// Requires permission - Android: <uses-permission android:name="android.permission.READ_PHONE_STATE"/>
  /// Source - SimInfo: https://pub.dev/packages/sim_info
  static Future<dynamic> getLanguageJson() async {
    if (_lng == null) {
      //Flutter (Found):
      String localeLanguage = "", simLanguage = ""; //Return empty as fallback.

      try {
        localeLanguage = (window.locale.languageCode == null)
            ? ""
            : window.locale.languageCode; // for example: en
      } catch (e) {
        Logger.logException(
            "DeviceDescriptor | geLanguageJson() | failed. e = ${e.toString()}");
      }

      //SimInfo package throws exception when it can't find data, due to:
      // - Publisher permission missing.
      // - User not accepting permission.
      // - Sim not present on device.
      try {
        simLanguage =
            await pluginSimInfo.SimInfo.getIsoCountryCode; // for example: il
      } catch (e) {
        Logger.logException(
            "DeviceDescriptor | geLanguageJson() | failed. e = ${e.toString()}");
      }

      _lng = {
        "main": localeLanguage,
        "locale": localeLanguage,
        "dsplng": "",
        "sim": simLanguage,
        "netw": "",
        "kbd": []
      };
    }

    return _lng;
  }

  static Future<String> getAdvertisingId() async {
    var advId;
    try {
      advId = TaboolaSdkPlugin.getAdIdOrIdfa();
    } catch (e) {
      Logger.logException(e.toString());
    }
    return advId == null ? "" : advId;
  }
}

==> taboola_sdk-0.3.3.tar.gz: lib/descriptors/device_descriptor.dart <==
import 'dart:io' as dartIo;
import 'dart:ui';
import 'dart:convert';
import 'package:taboola_sdk/utils/logger.dart';
import 'package:device_info_plus/device_info_plus.dart';
import 'package:system_info2/system_info2.dart';
import 'package:taboola_sdk/utils/text_utils.dart';
import 'package:taboola_sdk/native_plugin/taboola_sdk_plugin.dart';

class DeviceDescriptor {
  static const _undefinedValue = "undefined";
  static DeviceInfoPlugin _deviceInfoPlugin = DeviceInfoPlugin();

  //These fields will hold references to device data, allowing memory cache of results
  //TODO: If product leaves MVP, caching in app storage will be better for some values.
  static String _model = ""; //the device’s model string
  static var _lng;
  static String _os = ""; //“Android” or “iOS”
  static String _osv = ""; //the OS version
  static String _mem = ""; //the device current available memory
  static List? _scr; //screen sized (height and width)

  static Future<String> getDeviceModel() async {
    //Note: Taboola Android SDK runs a process on BuildConfig.Model before returning its value, will be simulated here.
    if (TextUtils.isEmptyOrNull(_model)) {
      if (dartIo.Platform.isAndroid) {
        AndroidDeviceInfo androidDeviceInfo =
            await _deviceInfoPlugin.androidInfo;
        String? nativeModel = androidDeviceInfo.model;
        String? nativeManufacturer = androidDeviceInfo.manufacturer;
        Pattern pattern = RegExp(nativeManufacturer ??
            ''); // convert to non-null string and create pattern

        //Some devices start with
        if (nativeModel != null && nativeModel.startsWith(pattern)) {
          _model = TextUtils.capitalize(nativeModel);
        } else {
          _model =
              '${TextUtils.capitalize(nativeManufacturer ?? "")} $nativeModel';
        }

        //Html escape value
        _model = HtmlEscape().convert(_model);
      }

      //Note: iOS returns model only, as manufacturer is always Epel. .. .
      if (dartIo.Platform.isIOS) {
        IosDeviceInfo iosDeviceInfo = await _deviceInfoPlugin.iosInfo;
        _model = iosDeviceInfo.model ?? "";
      }
    }

    return _model;
  }

  static Future<String> getDeviceOS() async {
    if (TextUtils.isEmptyOrNull(_os)) {
      if (dartIo.Platform.isAndroid) {
        _os = "Android";
      }

      if (dartIo.Platform.isIOS) {
        _os = "iOS";
      }
    }
    return _os;
  }

  /// Uses plugin "device_info"
  /// Source: https://stackoverflow.com/questions/45300661/how-to-check-the-device-os-version-from-flutter
  static Future<String> getDeviceSystemVersion() async {
    if (TextUtils.isEmptyOrNull(_osv)) {
      if (dartIo.Platform.isAndroid) {
        AndroidDeviceInfo androidDeviceInfo =
            await _deviceInfoPlugin.androidInfo;
        _osv = androidDeviceInfo.version.release ?? "";
      }

      if (dartIo.Platform.isIOS) {
        IosDeviceInfo iosDeviceInfo = await _deviceInfoPlugin.iosInfo;
        _osv = iosDeviceInfo.systemVersion ?? "";
      }
    }
    return _osv;
  }

  ///TODO: Restore in the future, requires either ripping just relevant code from Connectivity plugin (preferred solution) or by asking publishers to add two properties, describing usage of location, to their info.plist: 'NSLocationAlwaysAndWhenInUseUsageDescription' & 'NSLocationWhenInUseUsageDescription'.  Source:  https://pub.dev/packages/connectivity
//  /// Uses plugin "connectivity".
//  /// Existing Android/iOS values are: "wifi", "2g", "3g", "4g"
//  ///
//  /// NOTE: Flutter, incl. Connectivity package currently only provide wifi or cell. No '2g/3g/4g/..' resolution.
//  ///       A PR is open for adding the 'networkType' resolution but still wasn't accepted: https://github.com/flutter/plugins/pull/1945
//  ///
//  /// NOTE: Unlike other values here, every query to this method must pull current results as values are dynamic.
//  ///
//  /// Source: https://pub.dev/packages/connectivity
//  /// Source: https://pub.dev/documentation/connectivity/latest/
//  static Future<String> getNetworkConnSpeed() async {
//      var connectivityResult = await (Connectivity().checkConnectivity());
//      if (connectivityResult == ConnectivityResult.mobile) {
//        return "3g"; //Providing Taboola back-end with common value until better resolution is available.
//      } else if (connectivityResult == ConnectivityResult.wifi) {
//        return "wifi";
//      }
//
//      return "offline"; //value taken from iOS, Android SDK does not handle this case.
//  }

  /// Source: https://pub.dev/packages/system_info
  static Future<String> getDeviceFreeMemory() async {
    const int kiloByte = 1024;

    if (TextUtils.isEmptyOrNull(_mem)) {
      int memInteger;

      if (dartIo.Platform.isAndroid) {
        memInteger = SysInfo.getFreeVirtualMemory() ~/ kiloByte;
      } else {
        memInteger =
            0; //FIXME: Hardcoded, current version of library doesn't work on iOS properly.
      }

      _mem = "$memInteger";
    }
    return _mem;
  }

  /// Source - MediaQueryData: https://stackoverflow.com/questions/57052297/flutter-screen-size-calculation
  /// Source - window.physicalSize: https://stackoverflow.com/questions/49553402/flutter-screen-size
  static List? getScreenSize() {
    if (_scr == null) {
      double width = 0, height = 0;
      //Direct dart UI measurement
      Size windowSize = window.physicalSize;
      width = windowSize.width;
      height = windowSize.height;
      _scr = [width, height];
    }
    return _scr;
  }

  /// Language data isn't identical between what's currently available on Flutter and native Android/iOS.
  /// Agreed with product and team leader, at the moment, to provide what is possible and try and classify as closely as possible to Android/iOS values.
  ///
  /// NOTE: 'window.locales' returned an informative list ("[en_US, en, he_IL]") when I tried it, but it is officially known to change values for reasons I couldn't determine so not used right now.
  ///
  /// Requires permission - Android: <uses-permission android:name="android.permission.READ_PHONE_STATE"/>
  /// Source - SimInfo: https://pub.dev/packages/sim_info
  static Future<dynamic> getLanguageJson() async {
    if (_lng == null) {
      //Flutter (Found):
      String localeLanguage = "", simLanguage = ""; //Return empty as fallback.

      try {
        localeLanguage = (window.locale.languageCode == null)
            ? ""
            : window.locale.languageCode; // for example: en
      } catch (e) {
        Logger.logException(
            "DeviceDescriptor | geLanguageJson() | failed. e = ${e.toString()}");
      }

      //SimInfo package throws exception when it can't find data, due to:
      // - Publisher permission missing.
      // - User not accepting permission.
      // - Sim not present on device.
      try {
        simLanguage =
            await TaboolaSdkPlugin.getIsoCountryCode(); // for example: il
      } catch (e) {
        Logger.logException(
            "DeviceDescriptor | geLanguageJson() | failed. e = ${e.toString()}");
      }

      _lng = {
        "main": localeLanguage,
        "locale": localeLanguage,
        "dsplng": "",
        "sim": simLanguage,
        "netw": "",
        "kbd": []
      };
    }

    return _lng;
  }

  static Future<String> getAdvertisingId() async {
    var advId;
    try {
      advId = TaboolaSdkPlugin.getAdIdOrIdfa();
    } catch (e) {
      Logger.logException(e.toString());
    }
    return advId == null ? "" : advId;
  }
}
